package be.iminds.ilabt.jfed.highlevel.tasks;

import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.highlevel.controller.JFedHighLevelException;
import be.iminds.ilabt.jfed.highlevel.controller.Task;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.model.Slice;
import be.iminds.ilabt.jfed.highlevel.util.AggregateManagerWrapperFactory;
import be.iminds.ilabt.jfed.lowlevel.connection.JFedException;
import be.iminds.ilabt.jfed.lowlevel.api.user_spec.UserSpec;
import be.iminds.ilabt.jfed.lowlevel.api_wrapper.AggregateManagerWrapper;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * User: twalcari
 * Date: 6/17/2015
 * Time: 16:36
 */
public class EditSSHKeysTask extends Task {
    //input parameters
    @Nonnull private final Slice slice;
    @Nonnull private final Server auth;
    @Nonnull private final Collection<UserSpec> userSpecs;

    //environment parameters
    @Nonnull private final AggregateManagerWrapperFactory aggregateManagerWrapperFactory;
    @Nonnull private final HighLevelTaskFactory highLevelTaskFactory;

    public EditSSHKeysTask(@Nonnull Slice slice,
                           @Nonnull Server auth,
                           @Nonnull Collection<UserSpec> userSpecs,
                           @Nonnull AggregateManagerWrapperFactory aggregateManagerWrapperFactory,
                           @Nonnull HighLevelTaskFactory highLevelTaskFactory) {
        super("Edit SSH Keys");
        this.slice = slice;
        this.auth = auth;
        this.userSpecs = userSpecs;
        this.aggregateManagerWrapperFactory = aggregateManagerWrapperFactory;
        this.highLevelTaskFactory = highLevelTaskFactory;
    }

    @Override
    public void doTask(@Nonnull TaskExecution taskExecution) throws JFedException, InterruptedException {
        AggregateManagerWrapper amWrapper =
                aggregateManagerWrapperFactory.getAggregateManagerWrapper(taskExecution, auth);
        if (!amWrapper.isEditSshKeysSupported())
            throw new JFedHighLevelException("Authority " + auth.getDefaultComponentManagerUrn() + " does not support edit of SSH keys.");

        amWrapper.editSshKeys(slice.getUrn(), slice.getCredentials(), userSpecs);
    }

    @Nonnull
    @Override
    public List<Task> initDependsOn() {
        List<Task> res = new ArrayList<>();
        res.add(highLevelTaskFactory.getSliceCredential(slice));
        return res;
    }
}
